# ── FastMCP framework ───────────────────────────────────────────────
from mcp.server.fastmcp import FastMCP

mcp = FastMCP(
    name="tessell-mcp",
    version="0.1.0",
    description="Tessell DBaaS operations exposed via MCP"
)